/* eslint-disable @typescript-eslint/no-shadow */
import { useCallback, useRef, useState } from 'react';
import { isSectionList, isVirtualizedList } from './utils';
function getDataCountFromRef(listRef) {
  var _props$getItemCount, _props, _listRef$current$prop;
  if (!listRef.current) return 0;
  if (isVirtualizedList(listRef)) return ((_props$getItemCount = (_props = listRef.current.props).getItemCount) === null || _props$getItemCount === void 0 ? void 0 : _props$getItemCount.call(_props, null)) || 0;
  if (isSectionList(listRef)) return listRef.current.props.sections.reduce((count, item) => count + item.data.length, 0);
  return ((_listRef$current$prop = listRef.current.props.data) === null || _listRef$current$prop === void 0 ? void 0 : _listRef$current$prop.length) || 0;
}
export function usePagination(listRef, options) {
  const [pageIndex, setPageIndex] = useState(0);
  const [sectionItemIndex, setSectionItemIndex] = useState(0);
  const [sectionIndex, setSectionIndex] = useState(0);
  function scrollToIndex(params) {
    if (!(listRef !== null && listRef !== void 0 && listRef.current)) return;
    const dataCount = getDataCountFromRef(listRef);
    if (options.debugMode && dataCount === 0) return console.warn('Pagination does not work on empty lists.');
    let index;
    if (typeof params === 'number') index = params;else index = params.index;
    if (index < 0 || index >= dataCount) {
      if (options.loopPages) {
        index = (index + dataCount) % dataCount; // loop around
      } else {
        if (options.debugMode) console.warn(`Page: ${index} is outside bounderies of 0 - ${dataCount}.`);
        return;
      }
    }
    if ('scrollToIndex' in listRef.current) {
      setPageIndex(index);
      if (typeof params === 'number') listRef.current.scrollToIndex({
        index
      });else listRef.current.scrollToIndex({
        index,
        animated: params.animated,
        viewPosition: params.align
      });
      return;
    }
    if ('scrollToLocation' in listRef.current) {
      setPageIndex(index);
      let _index = index;
      let sectionIndex = 0;
      let itemIndex = 0;
      while (_index > 0) {
        var _listRef$current$prop2;
        itemIndex++;
        const sectionICount = ((_listRef$current$prop2 = listRef.current.props.sections[sectionIndex]) === null || _listRef$current$prop2 === void 0 ? void 0 : _listRef$current$prop2.data.length) || 0;
        if (itemIndex >= sectionICount) {
          sectionIndex++;
          itemIndex = 0;
        }
        _index--;
      }
      listRef.current.scrollToLocation({
        itemIndex,
        sectionIndex
      });
      setSectionItemIndex(itemIndex);
      setSectionIndex(sectionIndex);
      return;
    }
  }
  function nextPage(options) {
    scrollToIndex({
      index: pageIndex + 1,
      ...options
    });
  }
  function prevPage(options) {
    scrollToIndex({
      index: pageIndex - 1,
      ...options
    });
  }
  const viewabilityConfig = useRef({
    itemVisiblePercentThreshold: 50,
    waitForInteraction: true
  }).current;
  const isSectionSelected = useCallback(section => {
    if (!(listRef !== null && listRef !== void 0 && listRef.current)) return false;
    if (!('sections' in listRef.current.props)) return false;
    return listRef.current.props.sections.indexOf(section) === sectionIndex;
  }, [listRef, sectionIndex]);
  const onViewableItemsChanged = useCallback(_ref => {
    let {
      viewableItems
    } = _ref;
    const item = viewableItems[0];
    if (!item || item.index === null) return;
    if ('section' in item && listRef.current && 'sections' in listRef.current.props) {
      const sectionIndex = listRef.current.props.sections.indexOf(item.section);
      setSectionItemIndex(item.index);
      setSectionIndex(sectionIndex);
      let count = 0;
      for (let i = 0; i < sectionIndex; i++) {
        var _listRef$current$prop3;
        count += ((_listRef$current$prop3 = listRef.current.props.sections[i]) === null || _listRef$current$prop3 === void 0 ? void 0 : _listRef$current$prop3.data.length) || 0;
      }
      setPageIndex(count + item.index);
    } else {
      setPageIndex(item.index);
    }
  }, [listRef]);
  const indexController = useRef({
    viewabilityConfig,
    onViewableItemsChanged
  }).current;
  return {
    pageIndex,
    sectionItemIndex,
    sectionIndex,
    isSectionSelected,
    nextPage,
    prevPage,
    goToPage: scrollToIndex,
    indexController
  };
}
//# sourceMappingURL=usePagination.js.map